rm(list=ls())
setwd("~/Documents/Mike/Projects/Overlapping Clustering/Simulation/Final Code")
# source("DataGenerating.R")
source("Tuning.R")
source("EstNonpure.R")
source("EstPure.R")
source("Utilities.R")
source("LOVE.R")
source("Optimization.R")
# library(dplyr)

##### ------------------  Functions ------------------------- ######
# Helper function to see which variables have dominant variances
maxIndices = function(X)
{
  Sigma = t(X)%*%X/nrow(X)
  offdiag = Sigma - diag(diag(Sigma))
  return(table(apply(abs(offdiag),1,which.max)))
}
# collect result and change it to data.frame
collectResult = function(groupList,genNames)
{
  genName = varInd = group = dir = c()
  for (i in 1:length(groupList))
  {
    posgroupi = groupList[[i]][[1]]
    posLenGroupi = length(posgroupi)
    for (j in 1:posLenGroupi)
    {
      tmpVar = posgroupi[j]
      varInd = c(varInd,tmpVar)
      genName = c(genName,as.character(genNames[tmpVar]))
    }
    dir = c(dir,rep(1,posLenGroupi))
    neggroupi = groupList[[i]][[2]]
    negLenGroupi = length(neggroupi)
    group = c(group,rep(i,posLenGroupi+negLenGroupi))
    if (negLenGroupi != 0)
    {
      for (k in 1:negLenGroupi)
      {
        tmpVar = neggroupi[k]
        varInd = c(varInd,tmpVar)
        genName = c(genName,as.character(genNames[tmpVar]))
      }
      dir = c(dir,rep(-1,negLenGroupi))
    }
  }
  data = data.frame(name=as.factor(genName),index=varInd,group=group,dir=dir)
  return(data)
}
# calculate how many number of zero genes are clustered
calZeroGenes = function(zeroGenIndices,group)
{
  count = 0
  for (i in zeroGenIndices){
    if (checkElement(i,group)[1] != 0)
      count = count + 1
  }
  return(count)
}
# adapt Love
adaptive_LOVE = function(X,remInd,deltaGrids,mu=1,Union=T,diagonal=F,N=50)
{
  subX = X
  subX[,remInd] = 0
  # Tune delta
  if (length(deltaGrids)==1)
    optDelta = deltaGrids
  else
    optDelta = median(replicate(N,CV_Delta(subX,deltaGrids,Union,diagonal)))
  cat("Selecting the optimal delta =",optDelta,"\n")
  # Estimating AI
  subSigma = t(subX)%*%subX/nrow(subX)
  resultAI = EstAI(subSigma,optDelta,Union)
  AI = resultAI$AI
  # Estimating AJ
  Sigma = t(X)%*%X/nrow(X)
  C = EstC(Sigma,AI,diagonal); Y = EstY(Sigma,AI,resultAI$pureVec)
  if (diagonal)
  {
    C_inv = diag(diag(C)^{-1})
    cat("optDelta * max(C_inv) =",optDelta*max(diag(C_inv)),"\n")
    if (length(mu) == 1)
      optMu = mu*optDelta*max(diag(C_inv))
    else 
      optMu = mean(replicate(N,CV_OLS_thresh(X,mu*optDelta*max(diag(C_inv)),C_inv,AI,resultAI$pureVec)))
    cat("Selecting the optimal mu =",optMu,"\n")
    AJ = threshA(t(C_inv%*%Y),optMu)
  }else
  {
    Omega = solveOmega(optDelta,C)
    AJ = EstAJInv(Omega,Y,mu*optDelta*norm(Omega,"I"))
  }
  # Estimating clustering
  AI[-resultAI$pureVec,] = AJ;  group = recoverGroup(AI) 
  return(list(K=ncol(AI),A=AI,group=group,pureInd=resultAI$pureSignInd,C=C))
}

refindPureInd = function(A)
{
  pureList = vector("list",length=ncol(A))
  for (i in 1:nrow(A))
  {
    nonzeroInd = which(A[i,] != 0)
    if (length(nonzeroInd) == 1)
      pureList[[nonzeroInd]] = c(pureList[[nonzeroInd]],i)
  }
  return(pureList)
}

#### ------------------  Data pre-processing  --------------- ########
data = read.table("../Real data/Gene.txt",header=F)
rawX = data[,-1]; stand_rawX = scale(rawX,T,T)
### There are 308 genes whose entries are all 0. This implies only 192 out of 500
### genes have expression
n = nrow(rawX);  p = ncol(rawX)
# add noise to perturb the data
# set.seed(20170618)
# rawX = rawX + runif(n*p,0,0.1)
# centralize
X = scale(t(stand_rawX),T,F)
# record those dominant variables
maxIndices(X)
remInd = c(91,216,227,231,269,381,390,393)
maxIndices(X[,-remInd])
#X = X[,-remInd]
genNames = data[,1]
 
##### ------------------ Apply LOVE algorithm ----------------- ########
# Selecting the optimal delta
# deltaGrids = seq(0.001,0.01,0.002)*sqrt(log(p)/n)
# result = adaptive_LOVE(X,remInd,deltaGrids,diagonal=T,N=100)
# select delta = 0.0007383367
result = adaptive_LOVE(X,remInd,deltaGrids=0.0007383367,mu=1.1,diagonal=F,N=100)

##### --------------------- Post analysis ----------------------- #####
lengthresult = sapply(result$group,function(x){return(length(x[[1]])+length(x[[2]]))})
pureVar = refindPureInd(result$A)
sapply(pureVar,function(x){return(length(x))})
sapply(result$group,function(x){return(length(x[[1]])+length(x[[2]]))})
zeroGenIndices = which(apply(data[,-1],1,sum)==0)
cat("The number of clustered zero genes is",calZeroGenes(zeroGenIndices,result$group),"\n")
# write out the result
clusters = collectResult(result$group,genNames)
length(unique(clusters$name))
# write.csv(clusters,file="../Real data/gene_result.csv",row.names = F)

##### --------------------  Plotting  ----------------------- ####
library(GGally)
library(network)
library(sna)
library(viridis)
library(ggnetwork)


# function to create the edge-list data
convert_edge_list = function(A)
{
  # obtain transit matrix
  transit_count=transit_weight=matrix(0,nrow=12,ncol=12)
  for(j in 1:12){
    for(k in 1:12)
      transit_count[j,k]=sum(A[,j]>0&A[,k]>0)
  }
  lower_transit = transit_count*lower.tri(transit_count)
  output = c()
  for (i in 2:nrow(lower_transit))
  {
    for (j in 1:(i-1))
      output = rbind(output,c(i,j,lower_transit[i,j]))
  }
  colnames(output) = c("Node1","Node2","Count")
  output
}

edge_list = convert_edge_list(result$A)

net = network.edgelist(edge_list,g=network.initialize(ncol(result$A)),ignore.eval = F,names.eval = "weight")

# vertex names
network.vertex.names(net) =  paste("G",1:12,sep="")
net %v% "size" = sapply(result$group,function(x){return(length(x[[1]])+length(x[[2]]))})

ggnet2(net,mode ="fruchtermanreingold", layout.par = list(cell.jitter = 4.7),
       color="dodgerblue",size="size",size.legend = "Group size",
       edge.label = "weight",label = F,edge.label.fill = NA,edge.size = round(edge_list[,3]/15,2),
       edge.alpha = 0.5,edge.color = "tomato1",edge.label.alpha = 0.8,
       edge.label.size = 3.5,edge.label.color = "gray0")+
  geom_nodelabel_repel(aes(color = "dodgerblue", label = paste("G",1:12,sep="")),
                       fontface = "bold", box.padding = unit(1, "lines"))


sum(rowSums(abs(result$A))>0)
supp = c()
for (i in 1:(p-length(zeroGenIndices)))
{
  supp[i] = sum(result$A[-zeroGenIndices,][i,]!=0)
}
round(result$A[-zeroGenIndices,][supp==4,],2)
round(result$A[-zeroGenIndices,][supp==2,],2)
gen1 = genNames[-zeroGenIndices][which(supp==2)[7]]
gen2 = genNames[-zeroGenIndices][which(supp==4)[2]]
gen3 = genNames[-zeroGenIndices][which(supp==4)[6]]

selectedGenesInd = unique(clusters[clusters$name %in% c(as.character(gen1),as.character(gen2),as.character(gen3)),]$index)
selectedGenes = result$A[selectedGenesInd,]
rownames(selectedGenes) = genNames[selectedGenesInd]
round(selectedGenes,2)